package cz.cvut.fel.pr2;

/**
 * <p>
 *     The Stopwatch class provides convenience methods for time measuring. The motivation for static methods is a very
 *     simple usage. The class is <strong>not</strong> thread safe.
 * </p>
 *
 *  <p>
 *     The usage of this class is following. Right before measured operation, call the {@link Stopwatch#tick()} method.
 *     This will store the current timestamp. Right after measured operation, obtain the duration in milliseconds by
 *     calling the the {@link Stopwatch#tock()} method.
 * </p>
 *
 * <p>
 *     For example:
 *     <pre>
 *     {@code
 *     tick();
 *     measuredOperation();
 *     System.out.println("The operation took " + tock() +" ms of time");
 *     }
 *     </pre>
 * </p>
 *
 * <p>
 *     Note that this class measures time in milliseconds. It might not be precise enough for all cases of
 *     performance measurement. This class is intended only for demonstration purposes.
 * </p>
 *
 * @author Ondrej Hrstka
 */
public class Stopwatch {

    private static long lastTickTimeStamp = Long.MAX_VALUE;

    /**
     * Stores current timestamp. This method represents start of the stopwatch(). If the stopwatch is already
     * "running", the saved timestamp is rewritten.
     *
     * @see Stopwatch#tock()
     */
    public static void tick() {
        lastTickTimeStamp = System.currentTimeMillis();
    }

    /**
     * Gets number of milliseconds from last {@link Stopwatch#tick()} call and erases the stored timestamp. If there
     * wasn't call of {@link Stopwatch#tick()} before, the returned number is negative. Before another usage, there
     * must be another call to {@link Stopwatch#tick()} method.
     *
     * @return number of milliseconds from last {@link Stopwatch#tick()} or negative number if any call to
     * {@link Stopwatch#tick()} didn't happen.
     *
     * @see Stopwatch#tick()
     */
    public static long tock() {
        long out = System.currentTimeMillis() - lastTickTimeStamp;
        lastTickTimeStamp = Long.MAX_VALUE;
        return out;
    }

}
